/****************************************************************************

USB-CAN-Interface Communication Center
Dialog: Sendedatensatz editieren

(c) 2020 Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik

RADStudio XE7

Erstellt: 2020-03-11
Letzte nderung: 2020-03-11

*****************************************************************************/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UnitEditMsg.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TFormEditMsg *FormEditMsg;

//---------------------------------------------------------------------------
__fastcall TFormEditMsg::TFormEditMsg(TComponent* Owner)
	: TForm(Owner)
//---------------------------------------------------------------------------
{
	ClientWidth = BitBtnCancel->Left+BitBtnCancel->Width+BitBtnOk->Left;
	ClientHeight = BitBtnCancel->Top+BitBtnCancel->Height+BitBtnOk->Left;
	Edit[0] = Edit1;
	Edit[1] = Edit2;
	Edit[2] = Edit3;
	Edit[3] = Edit4;
	Edit[4] = Edit5;
	Edit[5] = Edit6;
	Edit[6] = Edit7;
	Edit[7] = Edit8;
}

//---------------------------------------------------------------------------
bool __fastcall TFormEditMsg::ExecuteNew(TPoint WinPos, TCANData *ACanData)
//---------------------------------------------------------------------------
{
	bool r = false;
	Left = WinPos.x;
	Top = WinPos.y;
	RadioGroup1->ItemIndex = 0;
	LabeledEditID->Clear();
	TabbedNotebook1->PageIndex = 0;
	for (int i=0; i<8; i++) Edit[i]->Clear();
	SpinEdit1->Value = 0;
	if (ShowModal()==mrOk) {
    DlgToData(ACanData);
		r = true;
	}
	return r;
}

//---------------------------------------------------------------------------
bool __fastcall TFormEditMsg::ExecuteEdit(TPoint WinPos, TCANData *ACanData)
//---------------------------------------------------------------------------
{
	bool r = false;
	Left = WinPos.x;
	Top = WinPos.y;
	if (ACanData->IdType==CAN_ID_STD) {
		RadioGroup1->ItemIndex = 0;
		LabeledEditID->Text = IntToHex((int)ACanData->Identifier,3);
	} else {
		RadioGroup1->ItemIndex = 1;
		LabeledEditID->Text = IntToHex((int)ACanData->Identifier,5);
	}
	int i;
	for (i=0; i<8; i++) Edit[i]->Clear();
	i = 0;
	while (i<ACanData->DataLength) {
		Edit[i]->Text = IntToHex((int)ACanData->Data[i],2);
		i++;
	}
	SpinEdit1->Value = ACanData->DataLength;
	TabbedNotebook1->PageIndex = (ACanData->RemoteRequest & CAN_RTR_REMOTE)>>1;
	if (ShowModal()==mrOk) {
		DlgToData(ACanData);
		r = true;
	}
	return r;
}

//---------------------------------------------------------------------------
uint32_t __fastcall TFormEditMsg::StrToHex(UnicodeString AStr)
//---------------------------------------------------------------------------
// Hex-String in Zahl umwandeln
{
	uint32_t r = 0, b;
	wchar_t c;
	if (AStr.Length()) {
		AStr = AStr.UpperCase();
		int j = 0;
		for (int i=1; i<=AStr.Length(); i++) {
			c = AStr[i];
			b = 0;
			if ((c>='0') && (c<='9')) b = uint32_t(c) & 0x0F;
			else
			if ((c>='A') && (c<='F')) b = (uint32_t(c) & 0xFF)-55;
			r = (r<<4)+b;
		}
	}
	return r;
}

//---------------------------------------------------------------------------
void __fastcall TFormEditMsg::DlgToData(TCANData *ACanData)
//---------------------------------------------------------------------------
// Eingaben in Datensatz bertragen
{
	const uint8_t IdValue[2] = {CAN_ID_STD,CAN_ID_EXT};
	ACanData->IdType = IdValue[RadioGroup1->ItemIndex];
	ACanData->Identifier = StrToHex(LabeledEditID->Text);
	if (ACanData->IdType==CAN_ID_STD) ACanData->Identifier = ACanData->Identifier & 0x7FF;
	else ACanData->Identifier = ACanData->Identifier & 0x1FFFFFFF;
	switch (TabbedNotebook1->PageIndex) {
	case 0:  // Daten
		ACanData->RemoteRequest = CAN_RTR_DATA;
		ACanData->DataLength = 0;
		while ((ACanData->DataLength<8) && (Edit[ACanData->DataLength]->Text.Length())) {
			ACanData->Data[ACanData->DataLength] = StrToHex(Edit[ACanData->DataLength]->Text);
			ACanData->DataLength++;
		}
		break;
	case 1:  // Remote Request
		ACanData->RemoteRequest = CAN_RTR_REMOTE;
		for (int i=0; i<8; i++) ACanData->Data[i] = 0;
		ACanData->DataLength = SpinEdit1->Value;
		break;
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormEditMsg::FormKeyUp(TObject *Sender, WORD &Key, TShiftState Shift)
//---------------------------------------------------------------------------
{
	if (Edit1->Focused() && (Edit1->Text.Length()>1)) Edit2->SetFocus();
	if (Edit2->Focused() && (Edit2->Text.Length()>1)) Edit3->SetFocus();
	if (Edit3->Focused() && (Edit3->Text.Length()>1)) Edit4->SetFocus();
	if (Edit4->Focused() && (Edit4->Text.Length()>1)) Edit5->SetFocus();
	if (Edit5->Focused() && (Edit5->Text.Length()>1)) Edit6->SetFocus();
	if (Edit6->Focused() && (Edit6->Text.Length()>1)) Edit7->SetFocus();
	if (Edit7->Focused() && (Edit7->Text.Length()>1)) Edit8->SetFocus();
	if (Edit8->Focused() && (Edit8->Text.Length()>1)) BitBtnOk->SetFocus();
}
//---------------------------------------------------------------------------

