/****************************************************************************

Basic class for user defined USB devices
based on LibUSB-Win32 driver

The interface to the USB driver wil be linked dynamically

(c) 2019-2020 Reusch Elektronik

Created: 2019-01-17
Changed: 2020-03-27

****************************************************************************/

#ifndef usbdeviceH
#define usbdeviceH

#include <System.Classes.hpp>
#include <Dialogs.hpp>
#include <Vcl.Forms.hpp>
#include <sysmac.h>
#include "usb.h" // special version rr

//---------------------------------------------------------------------------

class TUSBDevice
{
private:
	AnsiString FSerialNumber;
	bool IsReading, IsWriting;
	int FLastError;
	int FErrorCountWrite, FErrorCountRead;
protected:
	int FVendorID;              // Hersteller-Identifikation
	int FProductID;             // Produkt-Identifikation
	bool FOpened;								// true: Verbindung zum Gert hergestellt
	virtual int __fastcall Write(byte *Buffer, int Size, int Timeout);
	virtual int __fastcall Read(byte *Buffer, int Size, int Timeout);
public:
	struct usb_dev_handle *hdev; // USB device handle
	__property int LastError = { read=FLastError, write=FLastError };
	__property int ErrorCountWrite = { read=FErrorCountWrite, write=FErrorCountWrite };
	__property int ErrorCountRead = { read=FErrorCountRead, write=FErrorCountRead };
	__property bool Opened = { read=FOpened };
	__property int VendorID = { read=FVendorID };
	__property int ProductID = { read=FProductID };
	__property AnsiString SerialNumber = { read=FSerialNumber };
	__fastcall TUSBDevice(void);
	__fastcall ~TUSBDevice(void);
	virtual int __fastcall ScanForDevices(TStrings *DeviceList);
	virtual bool __fastcall DeviceRemoved(void);
	virtual uint32_t __fastcall CRC32WideFast(uint8_t *Buffer, unsigned int Size);
	virtual bool __fastcall Open(AnsiString SerialNo);
	virtual void __fastcall Close(void);
	virtual void __fastcall ResetIn(void);
};

bool USB_Initialize(void);
// Initialize USB communication
// Returns false, if Lib-USB driver is not installed.
// Function will be called automatically, if necessary.

#endif
